# Generated by Django 3.2.15 on 2022-08-18 04:57

import django.db.models.deletion
from django.db import connection, migrations, models

KEY_FIELD = '`key`' if connection.vendor == 'mysql' else 'key'


class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        ('django_rrweb', '0003_auto_20220818_0457'),
    ]

    operations = [
        migrations.RunSQL(
            f"""
            INSERT INTO django_rrweb_page
            SELECT id, create_time, {KEY_FIELD}, id AS session_id
            FROM django_rrweb_session
            """
        ),
        migrations.RunSQL(
            """
            UPDATE django_rrweb_event
            SET page_id = (
                SELECT id
                FROM django_rrweb_page
                WHERE django_rrweb_page.session_id = django_rrweb_event.session_id
            )
            """
        ),
        migrations.RemoveField(
            model_name='event',
            name='session',
        ),
        migrations.AlterField(
            model_name='event',
            name='page',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='events',
                to='django_rrweb.page',
            ),
        ),
    ]



# Generated by Django 3.2.15 on 2022-08-13 17:14

from django.db import migrations, models

import django_rrweb.models


class Migration(migrations.Migration):

    dependencies = [
        ('django_rrweb', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='session',
            name='key',
            field=models.CharField(
                default=django_rrweb.models._make_key,
                max_length=100,
                unique=True,
            ),
        ),
    ]



# Generated by Django 3.2.15 on 2022-08-09 04:49

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Session',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('create_time', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('kind', models.SmallIntegerField()),
                ('data', models.TextField(blank=True)),
                ('timestamp', models.BigIntegerField()),
                (
                    'session',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='events',
                        to='django_rrweb.session',
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 3.2.15 on 2022-08-18 04:57

import django.db.models.deletion
from django.db import migrations, models

import django_rrweb.models


class Migration(migrations.Migration):

    dependencies = [
        ('django_rrweb', '0002_session_key'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('create_time', models.DateTimeField(auto_now_add=True)),
                (
                    'key',
                    models.CharField(
                        default=django_rrweb.models._make_key,
                        max_length=100,
                        unique=True,
                    ),
                ),
                (
                    'session',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='pages',
                        to='django_rrweb.session',
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name='event',
            name='page',
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name='events',
                to='django_rrweb.page',
            ),
        ),
    ]



import datetime as dt
import json

from django.http import HttpResponse
from django.shortcuts import render
from django.utils import timezone as tz
from django.views.decorators.csrf import csrf_exempt

from .models import Event, Page, Session


@csrf_exempt
def record_events(request):
    """Record events"""
    if request.method != 'POST':
        return HttpResponse('ERROR: POST request required', status=405)
    obj = json.load(request)
    session, _ = Session.objects.get_or_create(key=obj['rrwebSessionKey'])
    page_key = obj['rrwebPageKey']
    page, _ = Page.objects.get_or_create(session=session, key=page_key)
    events = [
        Event(
            data=json.dumps(event['data']),
            kind=event['type'],
            page=page,
            timestamp=event['timestamp'],
        )
        for event in obj['rrwebEvents']
    ]
    Event.objects.bulk_create(events)
    return HttpResponse('OK', content_type='text/plain')


def record_script(request):
    """Record script"""
    session_key = request.session.get('django_rrweb_session_key')
    if session_key is None:
        session_key = Session.make_key()
        request.session['django_rrweb_session_key'] = session_key
    else:
        event = (
            Event.objects.filter(page__session__key=session_key)
            .order_by('-timestamp')
            .only('timestamp')
            .first()
        )
        delta = dt.timedelta(minutes=10)
        update = event is not None and (tz.now() - event.datetime) > delta
        if update:
            session_key = Session.make_key()
            request.session['django_rrweb_session_key'] = session_key
    context = {'page_key': Page.make_key(), 'session_key': session_key}
    return render(
        request,
        'django-rrweb/record-script.js',
        context,
        content_type='application/javascript',
    )



# Generated by Django 3.2.15 on 2022-08-18 23:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_rrweb', '0004_auto_20220818_0457'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='notes',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AddField(
            model_name='session',
            name='notes',
            field=models.TextField(blank=True, default=''),
        ),
    ]



import datetime as dt

from django.contrib import admin
from django.shortcuts import get_object_or_404, render
from django.urls import path, reverse
from django.utils.html import format_html

from .models import Event, Page, Session


def _three_sig_figs(num):
    scales = [
        (1e11, 1e9, 0, 'G'),
        (1e10, 1e9, 1, 'G'),
        (1e9, 1e9, 2, 'G'),
        (1e8, 1e6, 0, 'M'),
        (1e7, 1e6, 1, 'M'),
        (1e6, 1e6, 2, 'M'),
        (1e5, 1e3, 0, 'K'),
        (1e4, 1e3, 1, 'K'),
        (1e3, 1e3, 2, 'K'),
        (0, 1, 0, ''),
    ]
    for limit, scale, digits, magnitude in scales:
        if limit <= num:
            break
    return f'{round(num / scale, digits):.{digits}f}{magnitude}'


@admin.register(Event)
class EventAdmin(admin.ModelAdmin):
    list_display = [
        'id',
        'timestamp',
        'data_length',
        'kind',
        'page',
        'session',
    ]
    list_select_related = ['page', 'page__session']
    fields = list_display + ['data']
    readonly_fields = fields
    search_fields = ['page__key', 'page__session__key']
    ordering = ['-timestamp']

    def get_queryset(self, request):
        queryset = super().get_queryset(request).with_extras()
        return queryset

    @admin.display(
        description='Session',
        ordering='page__session__id',
    )
    def session(self, obj):
        return obj.page.session

    @admin.display(
        description='Data Length',
        ordering='data_length_num',
    )
    def data_length(self, obj):
        return _three_sig_figs(obj.data_length_num)


class EventMixin:
    @admin.display(
        description='Event Duration',
        ordering='event_duration_num',
    )
    def event_duration(self, obj):
        microseconds = obj.event_duration_num
        delta = dt.timedelta(seconds=microseconds / 1000)
        duration = str(delta)[:-3]
        return duration

    @admin.display(
        description='Event Count',
        ordering='event_count_num',
    )
    def event_count(self, obj):
        return _three_sig_figs(obj.event_count_num)

    @admin.display(
        description='Event Data Length',
        ordering='event_data_length_num',
    )
    def event_data_length(self, obj):
        return _three_sig_figs(obj.event_data_length_num)

    @admin.display(
        description='Event Timestamp Min',
        ordering='event_timestamp_min_num',
    )
    def event_timestamp_min(self, obj):
        if obj.event_timestamp_min_num is None:
            return '-'
        return Event.timestamp_to_datetime(obj.event_timestamp_min_num)

    @admin.display(
        description='Event Timestamp Max',
        ordering='event_timestamp_max_num',
    )
    def event_timestamp_max(self, obj):
        if obj.event_timestamp_max_num is None:
            return '-'
        return Event.timestamp_to_datetime(obj.event_timestamp_max_num)

    @admin.display(
        description='Events',
    )
    def link_events(self, obj):
        url = reverse('admin:django_rrweb_event_changelist')
        return format_html(
            '<a href="{url}?q={key}">events</a>', url=url, key=obj.key
        )


@admin.register(Page)
class PageAdmin(admin.ModelAdmin, EventMixin):
    list_display = [
        'key',
        'link_replay',
        'link_events',
        'create_time',
        'event_duration',
        'event_count',
        'event_data_length',
        'event_timestamp_min',
        'event_timestamp_max',
        'session',
    ]
    readonly_fields = list_display
    fields = readonly_fields + ['notes']
    ordering = ['-create_time']
    search_fields = ['key', 'session__key', 'notes']

    def get_queryset(self, request):
        queryset = super().get_queryset(request).with_extras()
        return queryset

    @admin.display(
        description='Replay',
    )
    def link_replay(self, obj):
        url = reverse(
            'admin:django-rrweb-page-replay',
            kwargs={'page_id': obj.id},
        )
        return format_html('<a href="{url}">replay</a>', url=url)

    def replay_view(self, request, page_id):
        page = get_object_or_404(Page, id=page_id)
        events = page.events.order_by('timestamp')
        context = dict(
            self.admin_site.each_context(request),
            events=events,
            page=page,
        )
        return render(request, 'django-rrweb/page-replay.html', context)

    def get_urls(self):
        urls = super().get_urls()
        replay_path = path(
            '<page_id>/replay/',
            self.replay_view,
            name='django-rrweb-page-replay',
        )
        urls.insert(0, replay_path)
        return urls


@admin.register(Session)
class SessionAdmin(admin.ModelAdmin, EventMixin):
    list_display = [
        'key',
        'link_pages',
        'link_events',
        'create_time',
        'page_count',
        'event_duration',
        'event_count',
        'event_data_length',
        'event_timestamp_min',
        'event_timestamp_max',
    ]
    readonly_fields = list_display
    fields = readonly_fields + ['notes']
    ordering = ['-create_time']
    search_fields = ['key', 'notes']

    def get_queryset(self, request):
        queryset = super().get_queryset(request).with_extras()
        return queryset

    @admin.display(
        description='Page Count',
        ordering='page_count_num',
    )
    def page_count(self, obj):
        return _three_sig_figs(obj.page_count_num)

    @admin.display(
        description='Pages',
    )
    def link_pages(self, obj):
        url = reverse('admin:django_rrweb_page_changelist')
        return format_html(
            '<a href="{url}?q={key}">pages</a>', url=url, key=obj.key
        )



import datetime as dt
import uuid

from django.db import models
from django.db.models import functions
from django.utils import timezone as tz


def _make_key():
    return str(uuid.uuid4()).replace('-', '')


class SessionQuerySet(models.QuerySet):
    def with_extras(self):
        queryset = (
            self.annotate(
                page_count_num=models.Count('pages__id', distinct=True)
            )
            .annotate(event_count_num=models.Count('pages__events__id'))
            .annotate(
                event_duration_num=functions.Coalesce(
                    models.Max('pages__events__timestamp')
                    - models.Min('pages__events__timestamp'),
                    0,
                )
            )
            .annotate(
                event_data_length_num=functions.Coalesce(
                    models.Sum(functions.Length('pages__events__data')), 0
                )
            )
            .annotate(
                event_timestamp_min_num=models.Min('pages__events__timestamp')
            )
            .annotate(
                event_timestamp_max_num=models.Max('pages__events__timestamp')
            )
        )
        return queryset


class Session(models.Model):
    create_time = models.DateTimeField(auto_now_add=True)
    key = models.CharField(default=_make_key, max_length=100, unique=True)
    notes = models.TextField(blank=True, default='')

    make_key = staticmethod(_make_key)

    objects = SessionQuerySet.as_manager()

    def __str__(self):
        return self.key


class PageQuerySet(models.QuerySet):
    def with_extras(self):
        queryset = (
            self.annotate(event_count_num=models.Count('events__id'))
            .annotate(
                event_duration_num=functions.Coalesce(
                    models.Max('events__timestamp')
                    - models.Min('events__timestamp'),
                    0,
                )
            )
            .annotate(
                event_data_length_num=functions.Coalesce(
                    models.Sum(functions.Length('events__data')), 0
                )
            )
            .annotate(event_timestamp_min_num=models.Min('events__timestamp'))
            .annotate(event_timestamp_max_num=models.Max('events__timestamp'))
        )
        return queryset


class Page(models.Model):
    create_time = models.DateTimeField(auto_now_add=True)
    key = models.CharField(default=_make_key, max_length=100, unique=True)
    session = models.ForeignKey(
        Session,
        on_delete=models.CASCADE,
        related_name='pages',
    )
    notes = models.TextField(blank=True, default='')

    make_key = staticmethod(_make_key)

    objects = PageQuerySet.as_manager()

    def __str__(self):
        return self.key


class EventQuerySet(models.QuerySet):
    def with_extras(self):
        queryset = self.annotate(data_length_num=functions.Length('data'))
        return queryset


class Event(models.Model):
    page = models.ForeignKey(
        Page,
        on_delete=models.CASCADE,
        related_name='events',
    )
    kind = models.SmallIntegerField()
    data = models.TextField(blank=True)
    timestamp = models.BigIntegerField()

    objects = EventQuerySet.as_manager()

    @property
    def datetime(self):
        return self.timestamp_to_datetime()

    def timestamp_to_datetime(self):
        timestamp = self.timestamp if isinstance(self, Event) else self
        datetime = tz.make_aware(dt.datetime.fromtimestamp(timestamp / 1000))
        return datetime

    def __str__(self):
        return f'#{self.id} @ {self.timestamp}'
